"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
const node_worker_threads_1 = require("node:worker_threads");
const configureProgramForMultiEmit_1 = require("./configureProgramForMultiEmit");
const typedWorkerData = node_worker_threads_1.workerData;
const ts = require(typedWorkerData.typeScriptToolPath);
function handleMessage(message) {
    if (!message) {
        process.exit(0);
    }
    try {
        const response = runTranspiler(message);
        node_worker_threads_1.parentPort.postMessage(response);
    }
    catch (err) {
        const errorResponse = {
            requestId: message.requestId,
            type: 'error',
            result: Object.assign({ message: err.message }, Object.fromEntries(Object.entries(err)))
        };
        node_worker_threads_1.parentPort.postMessage(errorResponse);
    }
}
function runTranspiler(message) {
    const { requestId, compilerOptions, moduleKindsToEmit, filesToTranspile } = message;
    const fullySkipTypeCheck = 
    /* TypeScript 5+ */ compilerOptions.verbatimModuleSyntax ||
        /* TypeScript 4 */ compilerOptions.importsNotUsedAsValues === ts.ImportsNotUsedAsValues.Error;
    for (const [option, value] of Object.entries(ts.getDefaultCompilerOptions())) {
        if (compilerOptions[option] === undefined) {
            compilerOptions[option] = value;
        }
    }
    const { target: rawTarget } = compilerOptions;
    for (const option of ts.transpileOptionValueCompilerOptions) {
        compilerOptions[option.name] = option.transpileOptionValue;
    }
    compilerOptions.suppressOutputPathCheck = true;
    compilerOptions.skipDefaultLibCheck = true;
    // To fully disable the type checker, we have to set `hasNoDefaultLib: true` on every source file.
    // However, doing so loses the information about which imports are used.
    // To retain the imports, we use `preserveValueImports`. However, if some imports are only used for types,
    // this will produce invalid runtime output unless said imports are marked with `type `.
    // Thus we can only enable this optimization if `verbatimModuleSyntax` is enabled (or equivalent).
    compilerOptions.preserveValueImports = fullySkipTypeCheck;
    const sourceFileByPath = new Map();
    const includedFiles = [];
    for (const [fileName, sourceText] of filesToTranspile) {
        if (sourceText) {
            const sourceFile = ts.createSourceFile(fileName, sourceText, rawTarget);
            sourceFile.hasNoDefaultLib = fullySkipTypeCheck;
            sourceFileByPath.set(fileName, sourceFile);
            includedFiles.push(fileName);
        }
    }
    const newLine = ts.getNewLineCharacter(compilerOptions);
    const compilerHost = {
        getSourceFile: (fileName) => sourceFileByPath.get(fileName),
        writeFile: ts.sys.writeFile,
        getDefaultLibFileName: () => 'lib.d.ts',
        useCaseSensitiveFileNames: () => true,
        getCanonicalFileName: (fileName) => fileName,
        getCurrentDirectory: () => '',
        getNewLine: () => newLine,
        fileExists: (fileName) => sourceFileByPath.has(fileName),
        readFile: () => '',
        directoryExists: () => true,
        getDirectories: () => []
    };
    const program = ts.createProgram(includedFiles, compilerOptions, compilerHost);
    (0, configureProgramForMultiEmit_1.configureProgramForMultiEmit)(program, ts, moduleKindsToEmit, 'transpile');
    const result = program.emit(undefined, 
    // The writeFile callback must be provided for the multi-emit redirector
    ts.sys.writeFile, undefined, undefined, undefined);
    const response = {
        requestId,
        type: 'success',
        result
    };
    return response;
}
node_worker_threads_1.parentPort.on('message', handleMessage);
//# sourceMappingURL=TranspilerWorker.js.map